@@ -23,9 +23,7 @@
 #endif
 
 #include <gst/gl/gl.h>
-#include <gst/gl/gstglfuncs.h>
 #if GST_GL_HAVE_PLATFORM_EGL && GST_GL_HAVE_DMABUF
-#include <gstgldmabufbufferpool.h>
 #include <gst/gl/egl/gsteglimage.h>
 #include <gst/allocators/gstdmabuf.h>
 #endif
@@ -863,7 +861,7 @@ gst_gl_download_element_class_init (GstGLDownloadElementClass * klass)
   gst_element_class_add_static_pad_template (element_class,
       &gst_gl_download_element_sink_pad_template);
 
-  gst_element_class_set_static_metadata (element_class,
+  gst_element_class_set_metadata (element_class,
       "OpenGL downloader", "Filter/Video",
       "Downloads data from OpenGL", "Matthew Waters <matthew@centricular.com>");
 
@@ -900,9 +898,6 @@ gst_gl_download_element_stop (GstBaseTransform * bt)
     dl->dmabuf_allocator = NULL;
   }
 
-  gst_clear_object (&dl->foreign_dmabuf_pool);
-  gst_clear_caps (&dl->foreign_dmabuf_caps);
-
   return TRUE;
 }
 
@@ -943,86 +938,16 @@ gst_gl_download_element_set_caps (GstBaseTransform * bt, GstCaps * in_caps,
   return TRUE;
 }
 
-#if GST_GL_HAVE_PLATFORM_EGL && GST_GL_HAVE_DMABUF
-static gboolean
-_convert_dma_drm (GstGLContext * context, GstStructure * s)
-{
-  const GValue *fmtval = gst_structure_get_value (s, "format");
-
-  if (!fmtval) {
-    return FALSE;
-  }
-
-  /* Convert from DMA_DRM to GstVideo formats */
-  if (G_VALUE_HOLDS_STRING (fmtval) &&
-      g_str_equal (g_value_get_string (fmtval), "DMA_DRM")) {
-    const GValue *drmval = gst_structure_get_value (s, "drm-format");
-    GValue newfmtval = G_VALUE_INIT;
-
-    if (context && gst_gl_dma_buf_transform_drm_formats_to_gst_formats (context,
-            drmval,
-            GST_GL_DRM_FORMAT_INCLUDE_EMULATED |
-            GST_GL_DRM_FORMAT_DIRECT_IMPORT, &newfmtval)) {
-      gst_structure_set_value (s, "format", &newfmtval);
-      gst_structure_remove_field (s, "drm-format");
-      g_value_unset (&newfmtval);
-    } else {
-      return FALSE;
-    }
-    /* Otherwise convert from GstVideo to DMA_DRM */
-  } else {
-    GValue drmfmtval = G_VALUE_INIT;
-
-    if (!context) {
-      gst_structure_remove_field (s, "drm-format");
-    } else if (gst_gl_dma_buf_transform_gst_formats_to_drm_formats (context,
-            fmtval, GST_GL_DRM_FORMAT_DIRECT_IMPORT, &drmfmtval)) {
-      gst_structure_set_value (s, "drm-format", &drmfmtval);
-      g_value_unset (&drmfmtval);
-    } else {
-      return FALSE;
-    }
-
-    gst_structure_set (s, "format", G_TYPE_STRING,
-        gst_video_format_to_string (GST_VIDEO_FORMAT_DMA_DRM), NULL);
-  }
-
-  return TRUE;
-}
-#endif
-
 static GstCaps *
-_set_caps_features (GstGLContext * context, const GstCaps * caps,
-    const gchar * feature_name)
+_set_caps_features (const GstCaps * caps, const gchar * feature_name)
 {
-  GstCaps *tmp = gst_caps_new_empty ();
-  guint n = gst_caps_get_size (caps);
+  GstCaps *tmp = gst_caps_copy (caps);
+  guint n = gst_caps_get_size (tmp);
   guint i = 0;
-#if GST_GL_HAVE_PLATFORM_EGL && GST_GL_HAVE_DMABUF
-  const gboolean new_feature_is_dmabuf =
-      g_str_equal (feature_name, GST_CAPS_FEATURE_MEMORY_DMABUF);
-#endif
 
-
-  for (i = 0; i < n; i++) {
-    GstStructure *s = gst_structure_copy (gst_caps_get_structure (caps, i));
-
-#if GST_GL_HAVE_PLATFORM_EGL && GST_GL_HAVE_DMABUF
-    gboolean old_feature_is_dmabuf =
-        gst_caps_features_contains (gst_caps_get_features (caps, i),
-        GST_CAPS_FEATURE_MEMORY_DMABUF);
-
-    if (new_feature_is_dmabuf != old_feature_is_dmabuf) {
-      if (!_convert_dma_drm (context, s)) {
-        gst_clear_structure (&s);
-        continue;
-      }
-    }
-#endif
-
-    gst_caps_append_structure_full (tmp, s,
-        gst_caps_features_new_single_static_str (feature_name));
-  }
+  for (i = 0; i < n; i++)
+    gst_caps_set_features (tmp, i,
+        gst_caps_features_from_string (feature_name));
 
   return tmp;
 }
@@ -1043,43 +968,32 @@ static GstCaps *
 gst_gl_download_element_transform_caps (GstBaseTransform * bt,
     GstPadDirection direction, GstCaps * caps, GstCaps * filter)
 {
-  GstGLBaseFilter *base_filter = GST_GL_BASE_FILTER (bt);
-
-  GstGLContext *context;
   GstCaps *result, *tmp;
 
-  if (base_filter->display && !gst_gl_base_filter_find_gl_context (base_filter))
-    return NULL;
-
-  context = gst_gl_base_filter_get_gl_context (base_filter);
-
   if (direction == GST_PAD_SRC) {
-    GstCaps *sys_caps = gst_caps_simplify (_set_caps_features (context, caps,
+    GstCaps *sys_caps = gst_caps_simplify (_set_caps_features (caps,
             GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY));
 
-    tmp = _set_caps_features (context, sys_caps,
-        GST_CAPS_FEATURE_MEMORY_GL_MEMORY);
+    tmp = _set_caps_features (sys_caps, GST_CAPS_FEATURE_MEMORY_GL_MEMORY);
     tmp = gst_caps_merge (tmp, sys_caps);
   } else {
     GstCaps *newcaps;
     tmp = gst_caps_ref (caps);
 
 #if GST_GL_HAVE_PLATFORM_EGL && defined(HAVE_NVMM)
-    newcaps = _set_caps_features (context, caps, GST_CAPS_FEATURE_MEMORY_NVMM);
+    newcaps = _set_caps_features (caps, GST_CAPS_FEATURE_MEMORY_NVMM);
     _remove_field (newcaps, "texture-target");
     // FIXME: RGBA-only?
     tmp = gst_caps_merge (tmp, newcaps);
 #endif
 
 #if GST_GL_HAVE_PLATFORM_EGL && GST_GL_HAVE_DMABUF
-    newcaps =
-        _set_caps_features (context, caps, GST_CAPS_FEATURE_MEMORY_DMABUF);
+    newcaps = _set_caps_features (caps, GST_CAPS_FEATURE_MEMORY_DMABUF);
     _remove_field (newcaps, "texture-target");
     tmp = gst_caps_merge (tmp, newcaps);
 #endif
 
-    newcaps = _set_caps_features (context, caps,
-        GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY);
+    newcaps = _set_caps_features (caps, GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY);
     _remove_field (newcaps, "texture-target");
     tmp = gst_caps_merge (tmp, newcaps);
   }
@@ -1091,9 +1005,6 @@ gst_gl_download_element_transform_caps (GstBaseTransform * bt,
     result = tmp;
   }
 
-  if (context)
-    gst_object_unref (context);
-
   GST_DEBUG_OBJECT (bt, "returning caps %" GST_PTR_FORMAT, result);
 
   return result;
@@ -1209,8 +1120,7 @@ _create_cached_dmabuf_info (GstGLContext * context, gpointer data)
 
       info = g_new0 (struct DmabufInfo, 1);
       info->dmabuf =
-          gst_dmabuf_allocator_alloc_with_flags (download->dmabuf_allocator,
-          fd, size, GST_FD_MEMORY_FLAG_KEEP_MAPPED);
+          gst_dmabuf_allocator_alloc (download->dmabuf_allocator, fd, size);
       info->stride = stride;
       info->offset = offset;
 
@@ -1230,7 +1140,6 @@ _try_export_dmabuf (GstGLDownloadElement * download, GstBuffer * inbuf)
   gsize offset[GST_VIDEO_MAX_PLANES];
   gint stride[GST_VIDEO_MAX_PLANES];
   GstCaps *src_caps;
-  GstVideoInfoDmaDrm out_info_drm;
   GstVideoInfo out_info;
   gsize total_offset;
   GstVideoAlignment *alig = NULL;
@@ -1243,11 +1152,6 @@ _try_export_dmabuf (GstGLDownloadElement * download, GstBuffer * inbuf)
     alig = &glmem->valign;
   }
 
-  src_caps = gst_pad_get_current_caps (GST_BASE_TRANSFORM (download)->srcpad);
-  if (!gst_video_info_dma_drm_from_caps (&out_info_drm, src_caps) ||
-      !gst_video_info_dma_drm_to_video_info (&out_info_drm, &out_info))
-    goto export_complete;
-
   buffer = gst_buffer_new ();
   total_offset = 0;
 
@@ -1283,6 +1187,10 @@ _try_export_dmabuf (GstGLDownloadElement * download, GstBuffer * inbuf)
     }
   }
 
+  src_caps = gst_pad_get_current_caps (GST_BASE_TRANSFORM (download)->srcpad);
+  gst_video_info_from_caps (&out_info, src_caps);
+  gst_caps_unref (src_caps);
+
   if (download->add_videometa) {
     GstVideoMeta *meta;
 
@@ -1309,7 +1217,6 @@ _try_export_dmabuf (GstGLDownloadElement * download, GstBuffer * inbuf)
   }
 
 export_complete:
-  gst_clear_caps (&src_caps);
 
   return buffer;
 }
@@ -1332,13 +1239,12 @@ gst_gl_download_element_prepare_output_buffer (GstBaseTransform * bt,
   in_sync_meta = gst_buffer_get_gl_sync_meta (inbuf);
   if (in_sync_meta) {
     if (context) {
-      gst_gl_sync_meta_wait_cpu (in_sync_meta, context);
+      gst_gl_sync_meta_wait (in_sync_meta, context);
     } else if (dl->mode != GST_GL_DOWNLOAD_MODE_PASSTHROUGH) {
       GST_WARNING_OBJECT (dl, "No configured GL context in non-passthrough "
           "mode. Cannot wait on incoming `GstGLSyncMeta`");
     }
   }
-
 #if GST_GL_HAVE_PLATFORM_EGL && defined(HAVE_NVMM)
   if (dl->mode == GST_GL_DOWNLOAD_MODE_NVMM) {
     GstNVMMParentMeta *buf_meta = gst_buffer_get_nvmm_parent_meta (inbuf);
@@ -1384,18 +1290,6 @@ gst_gl_download_element_prepare_output_buffer (GstBaseTransform * bt,
   }
 #endif
 #if GST_GL_HAVE_PLATFORM_EGL && GST_GL_HAVE_DMABUF
-  if (gst_is_gl_dmabuf_buffer (inbuf)) {
-    GstBuffer *wrapped_dmabuf = gst_gl_dmabuf_buffer_unwrap (inbuf);
-
-    if (wrapped_dmabuf) {
-      *outbuf = wrapped_dmabuf;
-
-      return GST_FLOW_OK;
-    } else {
-      GST_WARNING_OBJECT (dl, "could not unwrap dma-buf-backed GL buffer");
-    }
-  }
-
   if (dl->mode == GST_GL_DOWNLOAD_MODE_DMABUF_EXPORTS) {
     GstBuffer *buffer = _try_export_dmabuf (dl, inbuf);
 
@@ -1410,26 +1304,22 @@ gst_gl_download_element_prepare_output_buffer (GstBaseTransform * bt,
 
       *outbuf = buffer;
     } else {
-      GstCaps *sink_caps, *src_caps;
+      GstCaps *src_caps;
+      GstCapsFeatures *features;
       gboolean ret;
 
-      sink_caps = gst_pad_get_current_caps (bt->sinkpad);
-      src_caps = _set_caps_features (context, sink_caps,
-          GST_CAPS_FEATURE_MEMORY_SYSTEM_MEMORY);
-      _remove_field (src_caps, "texture-target");
-      gst_caps_unref (sink_caps);
-
+      src_caps = gst_pad_get_current_caps (bt->srcpad);
+      src_caps = gst_caps_make_writable (src_caps);
+      features = gst_caps_get_features (src_caps, 0);
+      gst_caps_features_remove (features, GST_CAPS_FEATURE_MEMORY_DMABUF);
       g_atomic_int_set (&dl->try_dmabuf_exports, FALSE);
       dl->mode = GST_GL_DOWNLOAD_MODE_PBO_TRANSFERS;
 
       ret = gst_base_transform_update_src_caps (bt, src_caps);
       gst_caps_unref (src_caps);
 
-      if (ret) {
-        GST_WARNING_OBJECT (bt, "DMABuf export didn't work. Falling back to "
-            "system memory.");
-      } else {
-        GST_ERROR_OBJECT (bt, "DMABuf export didn't work and system "
+      if (!ret) {
+        GST_ERROR_OBJECT (bt, "DMABuf exportation didn't work and system "
             "memory is not supported.");
         return GST_FLOW_NOT_NEGOTIATED;
       }
@@ -1461,31 +1351,13 @@ static gboolean
 gst_gl_download_element_transform_meta (GstBaseTransform * bt,
     GstBuffer * outbuf, GstMeta * meta, GstBuffer * inbuf)
 {
-  const GstMetaInfo *info = meta->info;
-  gboolean should_copy = TRUE;
-  const gchar *valid_tags[] = {
-    GST_META_TAG_VIDEO_STR,
-    GST_META_TAG_VIDEO_ORIENTATION_STR,
-    GST_META_TAG_VIDEO_SIZE_STR,
-    GST_META_TAG_VIDEO_COLORSPACE_STR,
-    NULL
-  };
-
   if (g_type_is_a (meta->info->api, GST_GL_SYNC_META_API_TYPE)) {
     GST_LOG_OBJECT (bt, "not copying GstGLSyncMeta onto output buffer");
     return FALSE;
   }
 
-  should_copy = gst_meta_api_type_tags_contain_only (info->api, valid_tags);
-
-  /* Cant handle the tags in this meta, let the parent class handle it */
-  if (!should_copy) {
-    return GST_BASE_TRANSFORM_CLASS (parent_class)->transform_meta (bt,
-        outbuf, meta, inbuf);
-  }
-
-  /* No need to transform, we can safely copy this meta */
-  return TRUE;
+  return GST_BASE_TRANSFORM_CLASS (parent_class)->transform_meta (bt, outbuf,
+      meta, inbuf);
 }
 
 static gboolean
@@ -1500,31 +1372,6 @@ gst_gl_download_element_decide_allocation (GstBaseTransform * trans,
     download->add_videometa = FALSE;
   }
 
-#if GST_GL_HAVE_PLATFORM_EGL && GST_GL_HAVE_DMABUF
-  {
-    GstCaps *caps;
-    const GstCapsFeatures *features;
-
-    gst_clear_object (&download->foreign_dmabuf_pool);
-    gst_clear_caps (&download->foreign_dmabuf_caps);
-
-    gst_query_parse_allocation (query, &caps, NULL);
-    features = gst_caps_get_features (caps, 0);
-
-    if (gst_caps_features_contains (features, GST_CAPS_FEATURE_MEMORY_DMABUF) &&
-        gst_query_get_n_allocation_pools (query) > 0) {
-
-      gst_query_parse_nth_allocation_pool (query, 0,
-          &download->foreign_dmabuf_pool, NULL, NULL, NULL);
-      GST_INFO_OBJECT (trans, "using foreign dmabuf pool %" GST_PTR_FORMAT,
-          download->foreign_dmabuf_pool);
-      download->foreign_dmabuf_caps = gst_caps_ref (caps);
-
-      gst_query_remove_nth_allocation_pool (query, 0);
-    }
-  }
-#endif
-
   return GST_BASE_TRANSFORM_CLASS (parent_class)->decide_allocation (trans,
       query);
 }
@@ -1597,18 +1444,6 @@ gst_gl_download_element_propose_allocation (GstBaseTransform * bt,
     }
   }
 #endif
-
-#if GST_GL_HAVE_PLATFORM_EGL && GST_GL_HAVE_DMABUF
-  if (!pool && GST_GL_DOWNLOAD_ELEMENT (bt)->foreign_dmabuf_pool) {
-    pool = gst_gl_dmabuf_buffer_pool_new (context,
-        GST_GL_DOWNLOAD_ELEMENT (bt)->foreign_dmabuf_pool,
-        GST_GL_DOWNLOAD_ELEMENT (bt)->foreign_dmabuf_caps);
-
-    GST_INFO_OBJECT (bt,
-        "offering dma-buf-backed GL buffer pool %" GST_PTR_FORMAT, pool);
-  }
-#endif
-
   if (!pool) {
     pool = gst_gl_buffer_pool_new (context);
   }
@@ -1617,16 +1452,13 @@ gst_gl_download_element_propose_allocation (GstBaseTransform * bt,
   /* the normal size of a frame */
   size = info.size;
   gst_buffer_pool_config_set_params (config, caps, size, 0, 0);
-  gst_buffer_pool_config_set_gl_min_free_queue_size (config, 1);
+  gst_buffer_pool_config_add_option (config,
+      GST_BUFFER_POOL_OPTION_GL_SYNC_META);
 
   if (!gst_buffer_pool_set_config (pool, config)) {
     gst_object_unref (pool);
     goto config_failed;
   }
-
-  if (context->gl_vtable->FenceSync)
-    gst_query_add_allocation_meta (query, GST_GL_SYNC_META_API_TYPE, NULL);
-
   gst_query_add_allocation_pool (query, pool, size, 1, 0);
 
   gst_object_unref (pool);
