@@ -833,8 +833,8 @@ gst_audio_channel_mixer_new_with_matrix (GstAudioChannelMixerFlags flags,
       || format == GST_AUDIO_FORMAT_S32
       || format == GST_AUDIO_FORMAT_F32
       || format == GST_AUDIO_FORMAT_F64, NULL);
-  g_return_val_if_fail (in_channels > 0 && in_channels < 64, NULL);
-  g_return_val_if_fail (out_channels > 0 && out_channels < 64, NULL);
+  g_return_val_if_fail (in_channels > 0 && in_channels <= 64, NULL);
+  g_return_val_if_fail (out_channels > 0 && out_channels <= 64, NULL);
 
   mix = g_slice_new0 (GstAudioChannelMixer);
   mix->in_channels = in_channels;
@@ -995,8 +995,8 @@ gst_audio_channel_mixer_new (GstAudioChannelMixerFlags flags,
       || format == GST_AUDIO_FORMAT_S32
       || format == GST_AUDIO_FORMAT_F32
       || format == GST_AUDIO_FORMAT_F64, NULL);
-  g_return_val_if_fail (in_channels > 0 && in_channels < 64, NULL);
-  g_return_val_if_fail (out_channels > 0 && out_channels < 64, NULL);
+  g_return_val_if_fail (in_channels > 0 && in_channels <= 64, NULL);
+  g_return_val_if_fail (out_channels > 0 && out_channels <= 64, NULL);
 
   matrix =
       gst_audio_channel_mixer_setup_matrix (flags, in_channels, in_position,
