@@ -27,6 +27,29 @@
 
 #include <gst/gststructure.h>
 
+#ifndef GST_DISABLE_GST_DEBUG
+#define GST_CAT_DEFAULT ensure_debug_category()
+static GstDebugCategory *
+ensure_debug_category (void)
+{
+  static gsize cat_gonce = 0;
+
+  if (g_once_init_enter (&cat_gonce)) {
+    gsize cat_done;
+
+    cat_done = (gsize) _gst_debug_category_new ("audio-info", 0,
+        "audio-info object");
+
+    g_once_init_leave (&cat_gonce, cat_done);
+  }
+
+  return (GstDebugCategory *) cat_gonce;
+}
+#else
+#define ensure_debug_category() /* NOOP */
+#endif /* GST_DISABLE_GST_DEBUG */
+
+
 /**
  * gst_audio_info_copy:
  * @info: a #GstAudioInfo
@@ -78,7 +101,7 @@ gst_audio_info_new (void)
 
 /**
  * gst_audio_info_init:
- * @info: a #GstAudioInfo
+ * @info: (out caller-allocates): a #GstAudioInfo
  *
  * Initialize @info with default values.
  */
@@ -98,7 +121,7 @@ gst_audio_info_init (GstAudioInfo * info)
  * @format: the format
  * @rate: the samplerate
  * @channels: the number of channels
- * @position: the channel positions
+ * @position: (array fixed-size=64) (nullable): the channel positions
  *
  * Set the default info for the audio info of @format and @rate and @channels.
  *
@@ -158,7 +181,7 @@ gst_audio_info_set_format (GstAudioInfo * info, GstAudioFormat format,
 
 /**
  * gst_audio_info_from_caps:
- * @info: a #GstAudioInfo
+ * @info: (out caller-allocates): a #GstAudioInfo
  * @caps: a #GstCaps
  *
  * Parse @caps and update @info.
@@ -170,13 +193,14 @@ gst_audio_info_from_caps (GstAudioInfo * info, const GstCaps * caps)
 {
   GstStructure *str;
   const gchar *s;
-  GstAudioFormat format;
-  gint rate, channels;
-  guint64 channel_mask;
+  GstAudioFormat format = GST_AUDIO_FORMAT_UNKNOWN;
+  gint rate = 0;
+  gint channels = 0;
+  guint64 channel_mask = 0;
   gint i;
   GstAudioChannelPosition position[64];
   GstAudioFlags flags;
-  GstAudioLayout layout;
+  GstAudioLayout layout = GST_AUDIO_LAYOUT_INTERLEAVED;
 
   g_return_val_if_fail (info != NULL, FALSE);
   g_return_val_if_fail (caps != NULL, FALSE);
@@ -188,28 +212,35 @@ gst_audio_info_from_caps (GstAudioInfo * info, const GstCaps * caps)
 
   str = gst_caps_get_structure (caps, 0);
 
-  if (!gst_structure_has_name (str, "audio/x-raw"))
+  if (gst_structure_has_name (str, "audio/x-raw")) {
+    if (!(s = gst_structure_get_string (str, "format")))
+      goto no_format;
+
+    format = gst_audio_format_from_string (s);
+    if (format == GST_AUDIO_FORMAT_UNKNOWN)
+      goto unknown_format;
+  } else if (g_str_has_prefix (gst_structure_get_name (str), "audio/")) {
+    format = GST_AUDIO_FORMAT_ENCODED;
+  } else {
     goto wrong_name;
+  }
 
-  if (!(s = gst_structure_get_string (str, "format")))
-    goto no_format;
+  if (format != GST_AUDIO_FORMAT_ENCODED) {
+    if (!(s = gst_structure_get_string (str, "layout")))
+      goto no_layout;
+    if (g_str_equal (s, "interleaved"))
+      layout = GST_AUDIO_LAYOUT_INTERLEAVED;
+    else if (g_str_equal (s, "non-interleaved"))
+      layout = GST_AUDIO_LAYOUT_NON_INTERLEAVED;
+    else
+      goto unknown_layout;
+  }
 
-  format = gst_audio_format_from_string (s);
-  if (format == GST_AUDIO_FORMAT_UNKNOWN)
-    goto unknown_format;
-
-  if (!(s = gst_structure_get_string (str, "layout")))
-    goto no_layout;
-  if (g_str_equal (s, "interleaved"))
-    layout = GST_AUDIO_LAYOUT_INTERLEAVED;
-  else if (g_str_equal (s, "non-interleaved"))
-    layout = GST_AUDIO_LAYOUT_NON_INTERLEAVED;
-  else
-    goto unknown_layout;
-
-  if (!gst_structure_get_int (str, "rate", &rate))
+  if (!gst_structure_get_int (str, "rate", &rate) &&
+      format != GST_AUDIO_FORMAT_ENCODED)
     goto no_rate;
-  if (!gst_structure_get_int (str, "channels", &channels))
+  if (!gst_structure_get_int (str, "channels", &channels) &&
+      format != GST_AUDIO_FORMAT_ENCODED)
     goto no_channels;
 
   if (!gst_structure_get (str, "channel-mask", GST_TYPE_BITMASK, &channel_mask,
@@ -219,7 +250,7 @@ gst_audio_info_from_caps (GstAudioInfo * info, const GstCaps * caps)
     } else if (channels == 2) {
       position[0] = GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT;
       position[1] = GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT;
-    } else {
+    } else if (format != GST_AUDIO_FORMAT_ENCODED) {
       goto no_channel_mask;
     }
   } else if (channel_mask == 0) {
@@ -289,6 +320,28 @@ invalid_channel_mask:
   }
 }
 
+/**
+ * gst_audio_info_new_from_caps:
+ * @caps: a #GstCaps
+ *
+ * Parse @caps to generate a #GstAudioInfo.
+ *
+ * Returns: (nullable): A #GstAudioInfo, or %NULL if @caps couldn't be parsed
+ * Since: 1.20
+ */
+GstAudioInfo *
+gst_audio_info_new_from_caps (const GstCaps * caps)
+{
+  GstAudioInfo *ret = gst_audio_info_new ();
+
+  if (gst_audio_info_from_caps (ret, caps)) {
+    return ret;
+  } else {
+    gst_audio_info_free (ret);
+    return NULL;
+  }
+}
+
 /**
  * gst_audio_info_to_caps:
  * @info: a #GstAudioInfo
@@ -376,7 +429,7 @@ invalid_channel_positions:
  * @src_fmt: #GstFormat of the @src_val
  * @src_val: value to convert
  * @dest_fmt: #GstFormat of the @dest_val
- * @dest_val: pointer to destination value
+ * @dest_val: (out): pointer to destination value
  *
  * Converts among various #GstFormat types.  This function handles
  * GST_FORMAT_BYTES, GST_FORMAT_TIME, and GST_FORMAT_DEFAULT.  For
