@@ -91,6 +91,18 @@
 #define DRM_FORMAT_RG1616       fourcc_code('R', 'G', '3', '2')
 #endif
 
+#ifndef DRM_FORMAT_ABGR2101010
+#define DRM_FORMAT_ABGR2101010    fourcc_code('A', 'B', '3', '0')
+#endif
+
+#ifndef DRM_FORMAT_RGBA1010102
+#define DRM_FORMAT_RGBA1010102    fourcc_code('R', 'A', '3', '0')
+#endif
+
+#ifndef DRM_FORMAT_ABGR16161616
+#define DRM_FORMAT_ABGR16161616   fourcc_code('A', 'B', '4', '8')
+#endif
+
 #ifndef EGL_LINUX_DMA_BUF_EXT
 #define EGL_LINUX_DMA_BUF_EXT 0x3270
 #endif
@@ -299,10 +311,8 @@ _gst_egl_image_create (GstGLContext * context, guint target,
   EGLContext egl_context = EGL_NO_CONTEXT;
   EGLImageKHR img = EGL_NO_IMAGE_KHR;
   GstGLDisplayEGL *display_egl;
-  gint plat_major, plat_minor;
   guint attrib_len = 0;
 
-  gst_gl_context_get_gl_platform_version (context, &plat_major, &plat_minor);
 
   display_egl = gst_gl_display_egl_from_gl_display (context->display);
   if (!display_egl) {
@@ -321,6 +331,9 @@ _gst_egl_image_create (GstGLContext * context, guint target,
     while (attribs[attrib_len++] != EGL_NONE) {
     }
 #ifdef EGL_VERSION_1_5
+  gint plat_major, plat_minor;
+  gst_gl_context_get_gl_platform_version (context, &plat_major, &plat_minor);
+
   if (GST_GL_CHECK_GL_VERSION (plat_major, plat_minor, 1, 5)) {
     EGLImageKHR (*gst_eglCreateImage) (EGLDisplay dpy, EGLContext ctx,
         EGLenum target, EGLClientBuffer buffer, const EGLAttrib * attrib_list);
@@ -347,7 +360,7 @@ _gst_egl_image_create (GstGLContext * context, guint target,
     g_free (egl_attribs);
   } else
 #endif
-  {
+  if (gst_gl_context_check_feature (context, "EGL_KHR_image_base")) {
     EGLImageKHR (*gst_eglCreateImageKHR) (EGLDisplay dpy, EGLContext ctx,
         EGLenum target, EGLClientBuffer buffer, const EGLint * attrib_list);
     EGLint *egl_attribs = NULL;
@@ -356,8 +369,8 @@ _gst_egl_image_create (GstGLContext * context, guint target,
     gst_eglCreateImageKHR = gst_gl_context_get_proc_address (context,
         "eglCreateImageKHR");
     if (!gst_eglCreateImageKHR) {
-      GST_WARNING_OBJECT (context, "\"eglCreateImageKHR\" not exposed by the "
-          "implementation");
+      GST_ERROR_OBJECT (context, "\"eglCreateImageKHR\" not exposed by the "
+          "implementation as required by EGL_KHR_image_base");
       return EGL_NO_IMAGE_KHR;
     }
 
@@ -371,6 +384,9 @@ _gst_egl_image_create (GstGLContext * context, guint target,
         egl_attribs);
 
     g_free (egl_attribs);
+  } else {
+    GST_INFO_OBJECT (context, "EGLImage creation not supported");
+    return EGL_NO_IMAGE_KHR;
   }
 
   return img;
@@ -383,16 +399,31 @@ _gst_egl_image_destroy (GstGLContext * context, EGLImageKHR image)
   EGLDisplay egl_display = EGL_DEFAULT_DISPLAY;
   GstGLDisplayEGL *display_egl;
 
-  gst_eglDestroyImage = gst_gl_context_get_proc_address (context,
-      "eglDestroyImage");
-  if (!gst_eglDestroyImage) {
+#ifdef EGL_VERSION_1_5
+  gint plat_major, plat_minor;
+  gst_gl_context_get_gl_platform_version (context, &plat_major, &plat_minor);
+
+  if (GST_GL_CHECK_GL_VERSION (plat_major, plat_minor, 1, 5)) {
+    gst_eglDestroyImage = gst_gl_context_get_proc_address (context,
+        "eglDestroyImage");
+    if (!gst_eglDestroyImage) {
+      GST_ERROR_OBJECT (context, "\"eglDestroyImage\" not exposed by the "
+          "implementation as required by EGL >= 1.5");
+      return;
+    }
+  } else
+#endif
+  if (gst_gl_context_check_feature (context, "EGL_KHR_image_base")) {
     gst_eglDestroyImage = gst_gl_context_get_proc_address (context,
         "eglDestroyImageKHR");
     if (!gst_eglDestroyImage) {
       GST_ERROR_OBJECT (context, "\"eglDestroyImage\" not exposed by the "
-          "implementation");
+          "implementation as required by EGL_KHR_image_base");
       return;
     }
+  } else {
+    GST_ERROR_OBJECT (context, "Destruction of EGLImage not supported.");
+    return;
   }
 
   display_egl = gst_gl_display_egl_from_gl_display (context->display);
@@ -421,7 +452,7 @@ _destroy_egl_image (GstEGLImage * image, gpointer user_data)
  * @gl_mem: a #GstGLMemory
  * @attribs: additional attributes to add to the `eglCreateImage`() call.
  *
- * Returns: (transfer full): a #GstEGLImage wrapping @gl_mem or %NULL on failure
+ * Returns: (transfer full) (nullable): a #GstEGLImage wrapping @gl_mem or %NULL on failure
  */
 GstEGLImage *
 gst_egl_image_from_texture (GstGLContext * context, GstGLMemory * gl_mem,
@@ -464,10 +495,14 @@ _drm_rgba_fourcc_from_info (const GstVideoInfo * info, int plane,
   const gint rgba_fourcc = DRM_FORMAT_ABGR8888;
   const gint rgb_fourcc = DRM_FORMAT_BGR888;
   const gint rg_fourcc = DRM_FORMAT_GR88;
+  const gint rg16_fourcc = DRM_FORMAT_GR1616;
+  const gint rgb10a2_fourcc = DRM_FORMAT_ABGR2101010;
 #else
   const gint rgba_fourcc = DRM_FORMAT_RGBA8888;
   const gint rgb_fourcc = DRM_FORMAT_RGB888;
   const gint rg_fourcc = DRM_FORMAT_RG88;
+  const gint rg16_fourcc = DRM_FORMAT_RG1616;
+  const gint rgb10a2_fourcc = DRM_FORMAT_RGBA1010102;
 #endif
 
   GST_DEBUG ("Getting DRM fourcc for %s plane %i",
@@ -510,6 +545,10 @@ _drm_rgba_fourcc_from_info (const GstVideoInfo * info, int plane,
 
     case GST_VIDEO_FORMAT_NV12:
     case GST_VIDEO_FORMAT_NV21:
+    case GST_VIDEO_FORMAT_NV16:
+    case GST_VIDEO_FORMAT_NV61:
+    case GST_VIDEO_FORMAT_NV12_16L32S:
+    case GST_VIDEO_FORMAT_NV12_4L4:
       *out_format = plane == 0 ? GST_GL_RED : GST_GL_RG;
       return plane == 0 ? DRM_FORMAT_R8 : rg_fourcc;
 
@@ -545,12 +584,45 @@ _drm_rgba_fourcc_from_info (const GstVideoInfo * info, int plane,
       *out_format = plane == 1 ? GST_GL_RED : GST_GL_RG;
       return plane == 1 ? rg_fourcc : DRM_FORMAT_R8;
 
+    case GST_VIDEO_FORMAT_Y210:
+      *out_format = GST_GL_RG16;
+      return rg16_fourcc;
+
+    case GST_VIDEO_FORMAT_Y212_LE:
+      *out_format = GST_GL_RG16;
+      return DRM_FORMAT_GR1616;
+
+    case GST_VIDEO_FORMAT_Y212_BE:
+      *out_format = GST_GL_RG16;
+      return DRM_FORMAT_RG1616;
+
+    case GST_VIDEO_FORMAT_Y410:
+      *out_format = GST_GL_RGB10_A2;
+      return rgb10a2_fourcc;
+
+    case GST_VIDEO_FORMAT_Y412_LE:
+      *out_format = GST_GL_RGBA16;
+      return DRM_FORMAT_ABGR16161616;
+
     default:
       GST_ERROR ("Unsupported format for DMABuf.");
       return -1;
   }
 }
 
+static gint
+get_egl_stride (const GstVideoInfo * info, gint plane)
+{
+  const GstVideoFormatInfo *finfo = info->finfo;
+  gint stride = info->stride[plane];
+
+  if (!GST_VIDEO_FORMAT_INFO_IS_TILED (finfo))
+    return stride;
+
+  return GST_VIDEO_TILE_X_TILES (stride) *
+      GST_VIDEO_FORMAT_INFO_TILE_STRIDE (finfo, plane);
+}
+
 /**
  * gst_egl_image_from_dmabuf:
  * @context: a #GstGLContext (must be an EGL context)
@@ -567,7 +639,7 @@ _drm_rgba_fourcc_from_info (const GstVideoInfo * info, int plane,
  * With NV12, two EGL images are created, one with R format, one
  * with RG format etc.
  *
- * Returns: a #GstEGLImage wrapping @dmabuf or %NULL on failure
+ * Returns: (nullable): a #GstEGLImage wrapping @dmabuf or %NULL on failure
  */
 GstEGLImage *
 gst_egl_image_from_dmabuf (GstGLContext * context,
@@ -599,7 +671,7 @@ gst_egl_image_from_dmabuf (GstGLContext * context,
   attribs[atti++] = EGL_DMA_BUF_PLANE0_OFFSET_EXT;
   attribs[atti++] = offset;
   attribs[atti++] = EGL_DMA_BUF_PLANE0_PITCH_EXT;
-  attribs[atti++] = GST_VIDEO_INFO_PLANE_STRIDE (in_info, plane);
+  attribs[atti++] = get_egl_stride (in_info, plane);
   attribs[atti] = EGL_NONE;
   g_assert (atti == G_N_ELEMENTS (attribs) - 1);
 
@@ -853,7 +925,7 @@ gst_egl_image_check_dmabuf_direct (GstGLContext * context,
  * is that this function creates one EGL image for all planes, not one for
  * a single plane.
  *
- * Returns: a #GstEGLImage wrapping @dmabuf or %NULL on failure
+ * Returns: (nullable): a #GstEGLImage wrapping @dmabuf or %NULL on failure
  *
  * Since: 1.18
  */
@@ -903,7 +975,7 @@ gst_egl_image_from_dmabuf_direct_target (GstGLContext * context,
     attribs[atti++] = EGL_DMA_BUF_PLANE0_OFFSET_EXT;
     attribs[atti++] = offset[0];
     attribs[atti++] = EGL_DMA_BUF_PLANE0_PITCH_EXT;
-    attribs[atti++] = in_info->stride[0];
+    attribs[atti++] = get_egl_stride (in_info, 0);
     if (with_modifiers) {
       attribs[atti++] = EGL_DMA_BUF_PLANE0_MODIFIER_LO_EXT;
       attribs[atti++] = DRM_FORMAT_MOD_LINEAR & 0xffffffff;
@@ -919,7 +991,7 @@ gst_egl_image_from_dmabuf_direct_target (GstGLContext * context,
     attribs[atti++] = EGL_DMA_BUF_PLANE1_OFFSET_EXT;
     attribs[atti++] = offset[1];
     attribs[atti++] = EGL_DMA_BUF_PLANE1_PITCH_EXT;
-    attribs[atti++] = in_info->stride[1];
+    attribs[atti++] = get_egl_stride (in_info, 1);
     if (with_modifiers) {
       attribs[atti++] = EGL_DMA_BUF_PLANE1_MODIFIER_LO_EXT;
       attribs[atti++] = DRM_FORMAT_MOD_LINEAR & 0xffffffff;
@@ -935,7 +1007,7 @@ gst_egl_image_from_dmabuf_direct_target (GstGLContext * context,
     attribs[atti++] = EGL_DMA_BUF_PLANE2_OFFSET_EXT;
     attribs[atti++] = offset[2];
     attribs[atti++] = EGL_DMA_BUF_PLANE2_PITCH_EXT;
-    attribs[atti++] = in_info->stride[2];
+    attribs[atti++] = get_egl_stride (in_info, 2);
     if (with_modifiers) {
       attribs[atti++] = EGL_DMA_BUF_PLANE2_MODIFIER_LO_EXT;
       attribs[atti++] = DRM_FORMAT_MOD_LINEAR & 0xffffffff;
@@ -1020,7 +1092,7 @@ gst_egl_image_from_dmabuf_direct_target (GstGLContext * context,
  * is that this function creates one EGL image for all planes, not one for
  * a single plane.
  *
- * Returns: a #GstEGLImage wrapping @dmabuf or %NULL on failure
+ * Returns: (nullable): a #GstEGLImage wrapping @dmabuf or %NULL on failure
  */
 GstEGLImage *
 gst_egl_image_from_dmabuf_direct (GstGLContext * context,
