@@ -469,10 +469,18 @@ gst_gl_window_wayland_egl_close (GstGLWindow * gl_window)
 
   destroy_surfaces (window_egl);
 
+  /* As we are about to destroy the wl_source, we need to ensure everything
+   * has been sent synchronously, otherwise we will be leaking surfaces on
+   * server, leaving the window visible and unrefreshed on screen. */
+  wl_display_flush (GST_GL_DISPLAY_WAYLAND (gl_window->display)->display);
+
   g_source_destroy (window_egl->wl_source);
   g_source_unref (window_egl->wl_source);
   window_egl->wl_source = NULL;
 
+  wl_proxy_wrapper_destroy (window_egl->display.display);
+  wl_event_queue_destroy (window_egl->window.queue);
+
   GST_GL_WINDOW_CLASS (parent_class)->close (gl_window);
 }
 
@@ -635,10 +643,6 @@ _roundtrip_async (GstGLWindow * window)
 static void
 gst_gl_window_wayland_egl_show (GstGLWindow * window)
 {
-  GstGLWindowWaylandEGL *window_egl = GST_GL_WINDOW_WAYLAND_EGL (window);
-
-  create_surfaces (window_egl);
-
   gst_gl_window_send_message (window, (GstGLWindowCB) _roundtrip_async, window);
 }
 
