@@ -35,7 +35,7 @@
 #include <gst/gl/gstglcontext.h>
 
 #include <gst/gl/gl.h>
-#include "gstglcontext_glx_private.h"
+#include "gstglcontext_glx.h"
 #include "../utils/opengl_versions.h"
 #include "../gstglcontext_private.h"
 
@@ -935,16 +935,15 @@ gst_gl_context_glx_get_config (GstGLContext * context)
 {
   GstGLContextGLX *glx = GST_GL_CONTEXT_GLX (context);
   GstGLWindow *window;
-  GstGLWindowX11 *window_x11 GST_UNUSED_CHECKS;
+  GstGLWindowX11 *window_x11;
   Display *device;
   GstStructure *ret;
 
   window = gst_gl_context_get_window (context);
   device = (Display *) gst_gl_display_get_handle (window->display);
-#ifndef G_DISABLE_CHECKS
   window_x11 = GST_GL_WINDOW_X11 (window);
+
   g_return_val_if_fail (glx->priv->fbconfigs || window_x11->visual_info, NULL);
-#endif
 
   if (glx->priv->fbconfigs) {
     ret = fb_config_to_structure (context, device, glx->priv->fbconfigs[0]);
@@ -1045,8 +1044,6 @@ gst_gl_context_glx_fill_info (GstGLContext * context, GError ** error)
       GST_GL_CONTEXT_WRAPPED_GL_CONFIG_NAME, config,
       (GDestroyNotify) gst_structure_free);
 
-  XFree (fbconfigs);
-
   return TRUE;
 
 failure:
