@@ -21,6 +21,7 @@
 
 /**
  * SECTION:gsttagexif
+ * @title: GstExiftag
  * @short_description: tag mappings and support functions for plugins
  *                     dealing with exif tags
  * @see_also: #GstTagList
@@ -608,7 +609,7 @@ gst_tag_list_has_ifd_tags (const GstTagList * taglist,
  * The tag entry is the tag id, the tag type,
  * the count and the offset.
  *
- * The offset is the on the amount of data writen so far, as one
+ * The offset is the on the amount of data written so far, as one
  * can't predict the total bytes that the tag entries will take.
  * This means those fields requires being updated later.
  */
@@ -1382,6 +1383,7 @@ parse_exif_undefined_tag (GstExifReader * reader, const GstExifTagMatch * tag,
 
   if (count > 4) {
     GstMapInfo info;
+    gsize alloc_size;
 
     if (offset < reader->base_offset) {
       GST_WARNING ("Offset is smaller (%u) than base offset (%u)", offset,
@@ -1403,14 +1405,28 @@ parse_exif_undefined_tag (GstExifReader * reader, const GstExifTagMatch * tag,
       return;
     }
 
+    if (info.size - real_offset < count) {
+      GST_WARNING ("Invalid size %u for buffer of size %" G_GSIZE_FORMAT
+          ", not adding tag %s", count, info.size, tag->gst_tag);
+      gst_buffer_unmap (reader->buffer, &info);
+      return;
+    }
+
+    if (!g_size_checked_add (&alloc_size, count, 1)) {
+      GST_WARNING ("Invalid size %u for buffer of size %" G_GSIZE_FORMAT
+          ", not adding tag %s", real_offset, info.size, tag->gst_tag);
+      gst_buffer_unmap (reader->buffer, &info);
+      return;
+    }
+
     /* +1 because it could be a string without the \0 */
-    data = malloc (sizeof (guint8) * count + 1);
+    data = malloc (alloc_size);
     memcpy (data, info.data + real_offset, count);
     data[count] = 0;
 
     gst_buffer_unmap (reader->buffer, &info);
   } else {
-    data = malloc (sizeof (guint8) * count + 1);
+    data = malloc (count + 1);
     memcpy (data, (guint8 *) offset_as_data, count);
     data[count] = 0;
   }
@@ -1501,6 +1517,9 @@ exif_reader_read_rational_tag (GstExifReader * exif_reader,
     }
   }
 
+  if (frac_d == 0)
+    return FALSE;               /* Ignore invalid fractions */
+
   if (_frac_n)
     *_frac_n = frac_n;
   if (_frac_d)
