@@ -138,6 +138,9 @@ static GParamSpec *pspec_add = NULL;
 
 #define gst_audio_rate_parent_class parent_class
 G_DEFINE_TYPE (GstAudioRate, gst_audio_rate, GST_TYPE_ELEMENT);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (audiorate, "audiorate", GST_RANK_NONE,
+    GST_TYPE_AUDIO_RATE, GST_DEBUG_CATEGORY_INIT (audio_rate_debug, "audiorate",
+        0, "AudioRate stream fixer"));
 
 static void
 gst_audio_rate_class_init (GstAudioRateClass * klass)
@@ -216,12 +219,24 @@ static gboolean
 gst_audio_rate_setcaps (GstAudioRate * audiorate, GstCaps * caps)
 {
   GstAudioInfo info;
+  gint prev_rate = 0;
 
   if (!gst_audio_info_from_caps (&info, caps))
     goto wrong_caps;
 
+  prev_rate = audiorate->info.rate;
   audiorate->info = info;
 
+  if (audiorate->next_offset >= 0 && prev_rate > 0 && prev_rate != info.rate) {
+    GST_DEBUG_OBJECT (audiorate,
+        "rate changed from %d to %d", prev_rate, info.rate);
+
+    /* calculate next_offset based on new rate value */
+    audiorate->next_offset =
+        gst_util_uint64_scale_int_round (audiorate->next_ts,
+        info.rate, GST_SECOND);
+  }
+
   return TRUE;
 
   /* ERRORS */
@@ -499,21 +514,9 @@ gst_audio_rate_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
   audiorate->in += in_samples;
 
   /* calculate the buffer offset */
-  if (in_time < audiorate->prev_in_time) {
-    in_offset = audiorate->prev_in_offset -
-        gst_util_uint64_scale_int_round (audiorate->prev_in_time - in_time,
-        rate, GST_SECOND);
-  } else {
-    in_offset =
-        audiorate->prev_in_offset +
-        gst_util_uint64_scale_int_round (in_time - audiorate->prev_in_time,
-        rate, GST_SECOND);
-  }
+  in_offset = gst_util_uint64_scale_int_round (in_time, rate, GST_SECOND);
   in_offset_end = in_offset + in_samples;
 
-  audiorate->prev_in_offset = in_offset;
-  audiorate->prev_in_time = in_time;
-
   GST_LOG_OBJECT (audiorate,
       "in_time:%" GST_TIME_FORMAT ", in_duration:%" GST_TIME_FORMAT
       ", in_size:%u, in_offset:%" G_GUINT64_FORMAT ", in_offset_end:%"
@@ -559,7 +562,7 @@ gst_audio_rate_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
       fill = gst_buffer_new_and_alloc (fillsize);
 
       gst_buffer_map (fill, &fillmap, GST_MAP_WRITE);
-      gst_audio_format_fill_silence (audiorate->info.finfo, fillmap.data,
+      gst_audio_format_info_fill_silence (audiorate->info.finfo, fillmap.data,
           fillmap.size);
       gst_buffer_unmap (fill, &fillmap);
 
@@ -578,8 +581,9 @@ gst_audio_rate_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
        * offset to get duration. Necessary complexity to get 'perfect' 
        * streams */
       GST_BUFFER_TIMESTAMP (fill) = audiorate->next_ts;
-      audiorate->next_ts +=
-          gst_util_uint64_scale_int_round (cursamples, GST_SECOND, rate);
+      audiorate->next_ts =
+          gst_util_uint64_scale_int_round (audiorate->next_offset, GST_SECOND,
+          rate);
       GST_BUFFER_DURATION (fill) =
           audiorate->next_ts - GST_BUFFER_TIMESTAMP (fill);
 
@@ -646,28 +650,27 @@ send:
   if (gst_buffer_get_size (buf) == 0)
     goto beach;
 
+  buf = gst_buffer_make_writable (buf);
+
   /* Now calculate parameters for whichever buffer (either the original
    * or truncated one) we're pushing. */
   GST_BUFFER_OFFSET (buf) = audiorate->next_offset;
   GST_BUFFER_OFFSET_END (buf) = in_offset_end;
 
   GST_BUFFER_TIMESTAMP (buf) = audiorate->next_ts;
-  audiorate->next_ts +=
-      gst_util_uint64_scale_int_round (in_offset_end - audiorate->next_offset,
+  audiorate->next_ts = gst_util_uint64_scale_int_round (in_offset_end,
       GST_SECOND, rate);
   GST_BUFFER_DURATION (buf) = audiorate->next_ts - GST_BUFFER_TIMESTAMP (buf);
 
   if (audiorate->discont) {
     /* we need to output a discont buffer, do so now */
     GST_DEBUG_OBJECT (audiorate, "marking DISCONT on output buffer");
-    buf = gst_buffer_make_writable (buf);
     GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
     audiorate->discont = FALSE;
   } else if (GST_BUFFER_IS_DISCONT (buf)) {
     /* else we make everything continuous so we can safely remove the DISCONT
      * flag from the buffer if there was one */
     GST_DEBUG_OBJECT (audiorate, "removing DISCONT from buffer");
-    buf = gst_buffer_make_writable (buf);
     GST_BUFFER_FLAG_UNSET (buf, GST_BUFFER_FLAG_DISCONT);
   }
 
@@ -782,11 +785,7 @@ gst_audio_rate_change_state (GstElement * element, GstStateChange transition)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  GST_DEBUG_CATEGORY_INIT (audio_rate_debug, "audiorate", 0,
-      "AudioRate stream fixer");
-
-  return gst_element_register (plugin, "audiorate", GST_RANK_NONE,
-      GST_TYPE_AUDIO_RATE);
+  return GST_ELEMENT_REGISTER (audiorate, plugin);
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
