@@ -36,12 +36,12 @@
 #include <string.h>
 
 #include <gst/gst.h>
-#include <gst/gst-i18n-plugin.h>
+#include <glib/gi18n-lib.h>
 #include <gst/pbutils/missing-plugins.h>
 
 #include "gstplay-enum.h"
 #include "gstrawcaps.h"
-#include "gstplayback.h"
+#include "gstplaybackelements.h"
 #include "gstplaybackutils.h"
 
 #define GST_TYPE_URI_DECODE_BIN \
@@ -119,6 +119,7 @@ struct _GstURIDecodeBin
   gboolean expose_allstreams;   /* Whether to expose unknown type streams or not */
 
   guint64 ring_buffer_max_size; /* 0 means disabled */
+  gboolean post_stream_topology;
 };
 
 struct _GstURIDecodeBinClass
@@ -186,6 +187,7 @@ enum
 #define DEFAULT_FORCE_SW_DECODERS   FALSE
 #define DEFAULT_EXPOSE_ALL_STREAMS  TRUE
 #define DEFAULT_RING_BUFFER_MAX_SIZE 0
+#define DEFAULT_POST_STREAM_TOPOLOGY FALSE
 
 enum
 {
@@ -201,7 +203,8 @@ enum
   PROP_USE_BUFFERING,
   PROP_FORCE_SW_DECODERS,
   PROP_EXPOSE_ALL_STREAMS,
-  PROP_RING_BUFFER_MAX_SIZE
+  PROP_RING_BUFFER_MAX_SIZE,
+  PROP_POST_STREAM_TOPOLOGY
 };
 
 static guint gst_uri_decode_bin_signals[LAST_SIGNAL] = { 0 };
@@ -210,6 +213,12 @@ GType gst_uri_decode_bin_get_type (void);
 #define gst_uri_decode_bin_parent_class parent_class
 G_DEFINE_TYPE (GstURIDecodeBin, gst_uri_decode_bin, GST_TYPE_BIN);
 
+#define _do_init \
+    GST_DEBUG_CATEGORY_INIT (gst_uri_decode_bin_debug, "uridecodebin", 0, "URI decoder element"); \
+    playback_element_init (plugin);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (uridecodebin, "uridecodebin",
+    GST_RANK_NONE, GST_TYPE_URI_DECODE_BIN, _do_init);
+
 static void remove_decoders (GstURIDecodeBin * bin, gboolean force);
 static void gst_uri_decode_bin_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -231,8 +240,7 @@ _gst_boolean_accumulator (GSignalInvocationHint * ihint,
   gboolean myboolean;
 
   myboolean = g_value_get_boolean (handler_return);
-  if (!(ihint->run_type & G_SIGNAL_RUN_CLEANUP))
-    g_value_set_boolean (return_accu, myboolean);
+  g_value_set_boolean (return_accu, myboolean);
 
   /* stop emission if FALSE */
   return myboolean;
@@ -248,8 +256,7 @@ _gst_boolean_or_accumulator (GSignalInvocationHint * ihint,
   myboolean = g_value_get_boolean (handler_return);
   retboolean = g_value_get_boolean (return_accu);
 
-  if (!(ihint->run_type & G_SIGNAL_RUN_CLEANUP))
-    g_value_set_boolean (return_accu, myboolean || retboolean);
+  g_value_set_boolean (return_accu, myboolean || retboolean);
 
   return TRUE;
 }
@@ -261,8 +268,7 @@ _gst_array_accumulator (GSignalInvocationHint * ihint,
   gpointer array;
 
   array = g_value_get_boxed (handler_return);
-  if (!(ihint->run_type & G_SIGNAL_RUN_CLEANUP))
-    g_value_set_boxed (return_accu, array);
+  g_value_set_boxed (return_accu, array);
 
   return FALSE;
 }
@@ -274,8 +280,7 @@ _gst_select_accumulator (GSignalInvocationHint * ihint,
   GstAutoplugSelectResult res;
 
   res = g_value_get_enum (handler_return);
-  if (!(ihint->run_type & G_SIGNAL_RUN_CLEANUP))
-    g_value_set_enum (return_accu, res);
+  g_value_set_enum (return_accu, res);
 
   /* Call the next handler in the chain (if any) when the current callback
    * returns TRY. This makes it possible to register separate autoplug-select
@@ -294,8 +299,7 @@ _gst_array_hasvalue_accumulator (GSignalInvocationHint * ihint,
   gpointer array;
 
   array = g_value_get_boxed (handler_return);
-  if (!(ihint->run_type & G_SIGNAL_RUN_CLEANUP))
-    g_value_set_boxed (return_accu, array);
+  g_value_set_boxed (return_accu, array);
 
   if (array != NULL)
     return FALSE;
@@ -463,7 +467,7 @@ gst_uri_decode_bin_class_init (GstURIDecodeBinClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstURIDecodeBin::download:
+   * GstURIDecodeBin:download:
    *
    * For certain media type, enable download buffering.
    */
@@ -472,7 +476,7 @@ gst_uri_decode_bin_class_init (GstURIDecodeBinClass * klass)
           "Attempt download buffering when buffering network streams",
           DEFAULT_DOWNLOAD, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
-   * GstURIDecodeBin::use-buffering:
+   * GstURIDecodeBin:use-buffering:
    *
    * Emit BUFFERING messages based on low-/high-percent thresholds of the
    * demuxed or parsed data.
@@ -486,7 +490,7 @@ gst_uri_decode_bin_class_init (GstURIDecodeBinClass * klass)
           DEFAULT_USE_BUFFERING, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstURIDecodeBin::force-sw-decoders:
+   * GstURIDecodeBin:force-sw-decoders:
    *
    * While auto-plugging, if set to %TRUE, those decoders within
    * "Hardware" klass will be ignored. Otherwise they will be tried.
@@ -500,7 +504,7 @@ gst_uri_decode_bin_class_init (GstURIDecodeBinClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstURIDecodeBin::expose-all-streams:
+   * GstURIDecodeBin:expose-all-streams:
    *
    * Expose streams of unknown type.
    *
@@ -516,7 +520,7 @@ gst_uri_decode_bin_class_init (GstURIDecodeBinClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GstURIDecodeBin::ring-buffer-max-size:
+   * GstURIDecodeBin:ring-buffer-max-size:
    *
    * The maximum size of the ring buffer in kilobytes. If set to 0, the ring
    * buffer is disabled. Default is 0.
@@ -528,6 +532,19 @@ gst_uri_decode_bin_class_init (GstURIDecodeBinClass * klass)
           0, G_MAXUINT, DEFAULT_RING_BUFFER_MAX_SIZE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstURIDecodeBin:post-stream-topology
+   *
+   * Post stream-topology messages on the bus every time the topology changes.
+   *
+   * Since: 1.22
+   */
+  g_object_class_install_property (gobject_class, PROP_POST_STREAM_TOPOLOGY,
+      g_param_spec_boolean ("post-stream-topology", "Post Stream Topology",
+          "Post stream-topology messages",
+          DEFAULT_POST_STREAM_TOPOLOGY,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   /**
    * GstURIDecodeBin::unknown-type:
    * @bin: The uridecodebin.
@@ -849,6 +866,9 @@ gst_uri_decode_bin_set_property (GObject * object, guint prop_id,
     case PROP_RING_BUFFER_MAX_SIZE:
       dec->ring_buffer_max_size = g_value_get_uint64 (value);
       break;
+    case PROP_POST_STREAM_TOPOLOGY:
+      dec->post_stream_topology = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -912,6 +932,9 @@ gst_uri_decode_bin_get_property (GObject * object, guint prop_id,
     case PROP_RING_BUFFER_MAX_SIZE:
       g_value_set_uint64 (value, dec->ring_buffer_max_size);
       break;
+    case PROP_POST_STREAM_TOPOLOGY:
+      g_value_set_boolean (value, dec->post_stream_topology);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1320,6 +1343,8 @@ static const gchar *adaptive_media[] = {
 
 /*
  * Generate and configure a source element.
+ *
+ * Returns: (transfer full): a new #GstElement
  */
 static GstElement *
 gen_source_element (GstURIDecodeBin * decoder)
@@ -1349,21 +1374,6 @@ gen_source_element (GstURIDecodeBin * decoder)
 
   GST_LOG_OBJECT (decoder, "found source type %s", G_OBJECT_TYPE_NAME (source));
 
-  decoder->is_stream = IS_STREAM_URI (decoder->uri);
-
-  query = gst_query_new_scheduling ();
-  if (gst_element_query (source, query)) {
-    gst_query_parse_scheduling (query, &flags, NULL, NULL, NULL);
-    if ((flags & GST_SCHEDULING_FLAG_BANDWIDTH_LIMITED))
-      decoder->is_stream = TRUE;
-  }
-  gst_query_unref (query);
-
-  GST_LOG_OBJECT (decoder, "source is stream: %d", decoder->is_stream);
-
-  decoder->need_queue = IS_QUEUE_URI (decoder->uri);
-  GST_LOG_OBJECT (decoder, "source needs queue: %d", decoder->need_queue);
-
   source_class = G_OBJECT_GET_CLASS (source);
 
   pspec = g_object_class_find_property (source_class, "connection-speed");
@@ -1410,6 +1420,34 @@ gen_source_element (GstURIDecodeBin * decoder)
         "setting subtitle-encoding=%s to source element", decoder->encoding);
     g_object_set (source, "subtitle-encoding", decoder->encoding, NULL);
   }
+
+  /* Sink reference before passing it to signal handler.
+   * Language binding might otherwise sink it and then unref.
+   * A floating ref is also tricky for a native signal handler in case
+   * of a "transfer floating" call followed by unref
+   * (e.g. some container_add and then container_remove).
+   * Bottom line; best not have a floating ref boldly going into unknown code.
+   */
+  g_object_ref_sink (source);
+
+  g_signal_emit (decoder, gst_uri_decode_bin_signals[SIGNAL_SOURCE_SETUP],
+      0, source);
+
+  decoder->is_stream = IS_STREAM_URI (decoder->uri);
+
+  query = gst_query_new_scheduling ();
+  if (gst_element_query (source, query)) {
+    gst_query_parse_scheduling (query, &flags, NULL, NULL, NULL);
+    if ((flags & GST_SCHEDULING_FLAG_BANDWIDTH_LIMITED))
+      decoder->is_stream = TRUE;
+  }
+  gst_query_unref (query);
+
+  GST_LOG_OBJECT (decoder, "source is stream: %d", decoder->is_stream);
+
+  decoder->need_queue = IS_QUEUE_URI (decoder->uri);
+  GST_LOG_OBJECT (decoder, "source needs queue: %d", decoder->need_queue);
+
   return source;
 
   /* ERRORS */
@@ -1462,23 +1500,23 @@ no_source:
 }
 
 /**
- * has_all_raw_caps:
+ * has_raw_caps:
  * @pad: a #GstPad
  * @all_raw: pointer to hold the result
  *
  * check if the caps of the pad are all raw. The caps are all raw if
  * all of its structures contain audio/x-raw or video/x-raw.
  *
- * Returns: %FALSE @pad has no caps. Else TRUE and @all_raw set t the result.
+ * Returns: %FALSE @pad caps are not set and raw
  */
 static gboolean
-has_all_raw_caps (GstPad * pad, GstCaps * rawcaps, gboolean * all_raw)
+has_raw_caps (GstPad * pad, GstCaps * rawcaps)
 {
   GstCaps *caps, *intersection;
   gint capssize;
   gboolean res = FALSE;
 
-  caps = gst_pad_query_caps (pad, NULL);
+  caps = gst_pad_get_current_caps (pad);
   if (caps == NULL)
     return FALSE;
 
@@ -1490,12 +1528,9 @@ has_all_raw_caps (GstPad * pad, GstCaps * rawcaps, gboolean * all_raw)
     goto done;
 
   intersection = gst_caps_intersect (caps, rawcaps);
-  *all_raw = !gst_caps_is_empty (intersection)
-      && (gst_caps_get_size (intersection) == capssize);
+  res = !gst_caps_is_empty (intersection);
   gst_caps_unref (intersection);
 
-  res = TRUE;
-
 done:
   gst_caps_unref (caps);
   return res;
@@ -1577,16 +1612,15 @@ analyse_source (GstURIDecodeBin * decoder, gboolean * is_raw,
         *have_out = TRUE;
 
         /* if FALSE, this pad has no caps and we continue with the next pad. */
-        if (!has_all_raw_caps (pad, rawcaps, is_raw)) {
+        if (!has_raw_caps (pad, rawcaps)) {
           gst_object_unref (pad);
           g_value_reset (&item);
           break;
-        }
-
-        /* caps on source pad are all raw, we can add the pad */
-        if (*is_raw) {
+        } else {
+          /* caps on source pad are all raw, we can add the pad */
           GstElement *outelem;
 
+          *is_raw = TRUE;
           if (use_queue) {
             GstPad *sinkpad;
 
@@ -1867,9 +1901,11 @@ make_decoder (GstURIDecodeBin * decoder)
   if (decoder->caps)
     g_object_set (decodebin, "caps", decoder->caps, NULL);
 
-  /* Propagate expose-all-streams and connection-speed properties */
+  /* Propagate expose-all-streams, connection-speed properties
+   * and post_stream_topology */
   g_object_set (decodebin, "expose-all-streams", decoder->expose_allstreams,
-      "connection-speed", decoder->connection_speed / 1000, NULL);
+      "connection-speed", decoder->connection_speed / 1000,
+      "post-stream-topology", decoder->post_stream_topology, NULL);
 
   if (!decoder->is_stream || decoder->is_adaptive) {
     /* propagate the use-buffering property but only when we are not already
@@ -2153,7 +2189,6 @@ static void
 source_new_pad (GstElement * element, GstPad * pad, GstURIDecodeBin * bin)
 {
   GstElement *decoder;
-  gboolean is_raw;
   GstCaps *rawcaps;
   GstPad *sinkpad;
 
@@ -2165,8 +2200,10 @@ source_new_pad (GstElement * element, GstPad * pad, GstURIDecodeBin * bin)
   if (!rawcaps)
     rawcaps = DEFAULT_CAPS;
 
-  /* if this is a pad with all raw caps, we can expose it */
-  if (has_all_raw_caps (pad, rawcaps, &is_raw) && is_raw) {
+  /* if this is a pad with all raw caps, we can expose it.
+   * But if stream-topology needs to be posted, we need to
+   * plug a decodebin so it can build the topology for us to forward. */
+  if (!bin->post_stream_topology && has_raw_caps (pad, rawcaps)) {
     /* it's all raw, create output pads. */
     GST_URI_DECODE_BIN_UNLOCK (bin);
     gst_caps_unref (rawcaps);
@@ -2257,13 +2294,12 @@ setup_source (GstURIDecodeBin * decoder)
   /* state will be merged later - if file is not found, error will be
    * handled by the application right after. */
   gst_bin_add (GST_BIN_CAST (decoder), decoder->source);
+  /* bin now has a ref, but the local reference is not counted */
+  g_object_unref (decoder->source);
 
   /* notify of the new source used */
   g_object_notify (G_OBJECT (decoder), "source");
 
-  g_signal_emit (decoder, gst_uri_decode_bin_signals[SIGNAL_SOURCE_SETUP],
-      0, decoder->source);
-
   if (is_live_source (decoder->source))
     decoder->is_stream = FALSE;
 
@@ -2273,6 +2309,10 @@ setup_source (GstURIDecodeBin * decoder)
   /* stream admin setup */
   decoder->streams = g_hash_table_new_full (NULL, NULL, NULL, free_stream);
 
+  if (gst_element_set_state (source,
+          GST_STATE_READY) != GST_STATE_CHANGE_SUCCESS)
+    goto state_fail;
+
   /* see if the source element emits raw audio/video all by itself,
    * if so, we can create streams for the pads and be done with it.
    * Also check that is has source pads, if not, we assume it will
@@ -2336,6 +2376,12 @@ no_source:
     /* error message was already posted */
     return FALSE;
   }
+state_fail:
+  {
+    GST_ELEMENT_ERROR (decoder, CORE, FAILED,
+        (_("Source element can't be prepared")), (NULL));
+    return FALSE;
+  }
 invalid_source:
   {
     GST_ELEMENT_ERROR (decoder, CORE, FAILED,
@@ -2607,7 +2653,7 @@ decoder_query_duration_fold (const GValue * item, GValue * ret,
 
     gst_query_parse_duration (fold->query, NULL, &duration);
 
-    GST_DEBUG_OBJECT (item, "got duration %" G_GINT64_FORMAT, duration);
+    GST_DEBUG_OBJECT (pad, "got duration %" G_GINT64_FORMAT, duration);
 
     if (duration > fold->max)
       fold->max = duration;
@@ -2640,7 +2686,7 @@ decoder_query_position_fold (const GValue * item, GValue * ret,
 
     gst_query_parse_position (fold->query, NULL, &position);
 
-    GST_DEBUG_OBJECT (item, "got position %" G_GINT64_FORMAT, position);
+    GST_DEBUG_OBJECT (pad, "got position %" G_GINT64_FORMAT, position);
 
     if (position > fold->max)
       fold->max = position;
@@ -2720,7 +2766,7 @@ decoder_query_seeking_fold (const GValue * item, GValue * ret, QueryFold * fold)
     g_value_set_boolean (ret, TRUE);
     gst_query_parse_seeking (fold->query, NULL, &seekable, NULL, NULL);
 
-    GST_DEBUG_OBJECT (item, "got seekable %d", seekable);
+    GST_DEBUG_OBJECT (pad, "got seekable %d", seekable);
 
     if (fold->seekable)
       fold->seekable = seekable;
@@ -2749,7 +2795,7 @@ decoder_query_generic_fold (const GValue * item, GValue * ret, QueryFold * fold)
 
   if ((res = gst_pad_query (pad, fold->query))) {
     g_value_set_boolean (ret, TRUE);
-    GST_DEBUG_OBJECT (item, "answered query %p", fold->query);
+    GST_DEBUG_OBJECT (pad, "answered query %p", fold->query);
   }
 
   /* and stop as soon as we have a valid result */
@@ -2931,13 +2977,3 @@ setup_failed:
     return GST_STATE_CHANGE_FAILURE;
   }
 }
-
-gboolean
-gst_uri_decode_bin_plugin_init (GstPlugin * plugin)
-{
-  GST_DEBUG_CATEGORY_INIT (gst_uri_decode_bin_debug, "uridecodebin", 0,
-      "URI decoder element");
-
-  return gst_element_register (plugin, "uridecodebin", GST_RANK_NONE,
-      GST_TYPE_URI_DECODE_BIN);
-}
